/* cc -Wall -o plbmerge -O2 plbmerge.c
   part of MACSIMUS
History:
  02/21: concatenation changed
  05/08: append option added
  06/05: new plb format (variable L[3])
  11/04: more frames allowed
  10/00: arbitrary selection added
  08/97: header[1] printed as t0 (10/00: changed into L)
         bug fixed (FROM did not work correctly)
         -BY added
  11/95: first version
*/

#include "../gen/include.h"

int main(int narg,char **arg)
{
  float r[3],hdr[2],L3[3];
  int i,iarg,NS=0;

  if (narg<2) {
    fprintf(stderr,"\
Merge binary playback files.  Call by:\n\
  plbmerge [-c[#]|-C] PLB-FILE [PLB-FILE ...] > MERGED-PLB-FILE\n\
OPTION:\n\
  NONE\tmerge frames synchronously (default):\n\
\t- stops when any plb-file reaches EOF\n\
\t- output box size L is set to MAXIMUM of all plb-files\n\
\t- hint: use molcfg to create merged mol- and gol-files\n\
  -cNS\ttruncate playback files to NS sites and concatenate\n\
  -c\tas above, the first file is the shortest (or all are of the same length)\n\
The output plb file contains box info (cf. plbconv or mergeplb for old format)\n\
Examples:\n\
  plbmerge -c run1.plb run2.plb > runs1+2.plb\n\
  plbmerge sim.p00 sim.p01 > sim.plb\n\
See also:\n\
  plbinfo plbmsd plbcut plbfilt plbtran plbrot plb2diff plbasc\n");
    exit(0); }

  hdr[1]=-3; /* always variable-box format on output */

  if (!memcmp(arg[1],"-c",2)) {
    /* concatenate */

    NS=atoi(arg[1]+1);
    
    loop (iarg,2,narg) {
      FILE *f=fopen(arg[iarg],"rb");

      if (!f) Error(arg[iarg]);

      if (2!=fread(r,4,2,f)) Error(arg[iarg]);
      fprintf(stderr,"%20s: ns=%-5.0f L/key=%f\n",arg[iarg],r[0],r[1]);
      if (r[1]<0) {
        if (3!=fread(L3,4,3,f)) Error(arg[iarg]); }
      else
        L3[0]=L3[1]=L3[2]=r[1];

      if (iarg==2) {
        if (NS==0) NS=r[0];
        else if (r[0]<NS) Error("1st file shorter than -cNS"); }

      if (r[0]<NS) Error("file shorter than -cNS");
      if (r[0]>NS) fprintf(stderr,"*** file will be truncated\n");

      if (iarg==2) {
        /* write header */
        hdr[0]=NS;
        fwrite(hdr,4,2,stdout); }
      
      fwrite(L3,4,3,stdout);

      loop (i,0,NS) {
        if (3!=fread(r,4,3,f)) Error(arg[iarg]);
        fwrite(r,4,3,stdout); }

      fclose (f); } }

  else {
    /* merge files synchronously */
    FILE **f;
    int *ns,*varL,frame=0,narg0=1,nsmin=0x7fffffff;
    float L=0,L3[3],*LL;
    
    hdr[0]=0;

    allocarray(f,narg);
    allocarray(ns,narg);
    allocarray(varL,narg);
    allocarray(LL,narg);

    loop (iarg,narg0,narg) {
      f[iarg]=fopen(arg[iarg],"rb");
      if (!f[iarg]) Error(arg[iarg]);
      if (2!=fread(r,4,2,f[iarg])) Error(arg[iarg]);
      fprintf(stderr,"%20s: ns=%-5.0f L/key=%f\n",arg[iarg],r[0],r[1]);
      varL[iarg]=r[1]<0;
      Max(L,r[1])
        LL[iarg]=r[1];
      Min(nsmin,(int)r[0])
        hdr[0]+=r[0];
      ns[iarg]=r[0]; }

    fprintf(stderr,"              OUTPUT: ns=%-5.0f key=%g (variable box)\n",hdr[0],hdr[1]);

    fwrite(hdr,4,2,stdout);

    for (;;) {
      L3[0]=L3[1]=L3[2]=L;
      loop (iarg,narg0,narg)
        if (varL[iarg]) {
          if (3!=fread(r,4,3,f[iarg])) {
            fprintf(stderr,"%d frame(s) written\n",frame);
            return 0; }
          Max(L3[0],r[0])
          Max(L3[1],r[1])
          Max(L3[2],r[2]) }

      fwrite(L3,4,3,stdout);

      loop (iarg,narg0,narg) {
        loop (i,0,ns[iarg]) {
          if (3!=fread(r,4,3,f[iarg])) {
            fprintf(stderr,"EOF on %s: %d frame(s) written\n",arg[iarg],frame);
            return 0; }
          fwrite(r,4,3,stdout); } }
      frame++; } }
  
  return 0;
}
