/* make spectrum
 */
#include "ground.h"
#include "fft.h"
/* float if -DFLOAT, otherwise double
   CAVEAT: not conforming to -DPRECISION */
typedef fftcomplex complex;
typedef fftreal real;
#include "solve2.h"

int n,win,prec=6;

double window(double x)
{
  x=sin(x/2);
  return powi(x,win);
}

double eps=1e-7,gpass;
int npass;
double A,*ypass;

double f(double a,double s)
{
  int i;
  double e,p,ee=0,ye=0,eepps=0,yepps=0,eepss=0,yepss=0;

  loopto (i,0,npass) {
    p=i-a;
    e=exp(-Sqr(p*s));
    ee+=e*e;
    ye+=ypass[i]*e;
    eepps+=e*e*Sqr(p*p)*s;
    yepps+=ypass[i]*e*Sqr(p*p)*s;
    eepss+=e*e*p*Sqr(s);
    yepss+=ypass[i]*e*p*Sqr(s); }

  A=ye/ee;
  gpass=A*eepss-yepss;

  return A*eepps-yepps;
}

double g(double a,double s)
{
  return gpass;
}

double peak(double *y,int n)
{
  double a=n/2,s=0.5;
  int err;

  npass=n;
  ypass=y;

  err=Solve2(f,g,
	     &a,0,n,eps,
	     &s,0,2,eps,-30,-2);

  if (err) return 0;
  else return a;
}

int isq=0,isverlet=0,iscm=1;
double dt=0;

double conv(double f)
{
  if (isverlet) {
    if (isq) f=sqrt(2-2*cos(PI*dt*f))/(2*PI*dt);
    else f=sqrt(2-2*cos(2*PI*dt*f))/(2*PI*dt); }
  if (iscm) f/=0.029979246;

  return f;
}

int main(int narg,char **arg)
{
  complex *y;
  real *sp;
  FILE *fpeak=NULL;
  char *fn=NULL;
  int iarg,ic,i,k=0,col=0,omit=0,plot=0,ndata=0,missing=0;
  double sum=0,slowdown=2;
  double minpeak=0;

  if (narg<2) {
    fprintf(stderr,"\
Spectrum (Fourier transform of data with optional window). Call by:\n\
  spectrum [OPTIONS] [FILE|-]\n\
FILES:\n\
  FILE = input file of white-separated data, #-, !-, empty lines are ignored\n\
  FILE.sp = output spectrum, if -h : 2 column (wavenumber,|amplitude|^2)\n\
                             if no -h : 1 column (|amplitude|^2)\n\
  FILE.pk = peaks analyzed (-h required)\n\
  -: input=stdin, both outputs are concatenated to stdout\n\
OPTIONS:\n\
  -c# = column of input data [default=1]\n\
  -d# = decimal digits for output g-format [default=6]\n\
  -e# = efficiency warning limit (if n=product of small primes) [default=2]\n\
  -h# = timestep in ps [default=1]\n\
  -k# = # of output frequences [default=n/2]\n\
  -m# = ignore peaks smaller than #; #<0: do not calculate peaks [default=0]\n\
  -n# = # of valid data lines on input [no default]\n\
        more data are ignored, less data are padded by the average\n\
  -o# = omit # lines of data from start [default=0]\n\
  -p  = plot the output spectrum (only if FILE given)\n\
  -q  = quantity is quadratically dependent on amplitude (as kin.energy)\n\
  -v  = apply Verlet frequency correction\n\
  -w# = multiply input data by bell-like window [sin(PI*(i+1/2)/NDATA)]^#\n\
        accuracy of peaks increases, #=1 or 2 recommended, large # -> Gaussian\n\
        WARNING: with -w, only a few data may be padded, cf. spectrum.sh\n\
  -T  = with -h: print frequncies in THz [default=wavenumbers in cm^-1]\n\
METHODS:\n\
  * fast Fourier transform with arbitrary factoring is used\n\
  * may be slow if the number of data is not a product of small primes:\n\
    - a warning is printed (use -eBIG to suppress)\n\
    - change -n: either omit some data or padd by zeros\n\
  * the peak searcher uses a parabola of 4 points closest to a maximum\n\
Examples:\n\
  tab 1 1024 | tabproc \"abs(cos(A))\" | spectrum -n1024 - | plot -:0:1\n\
  showcp -a -b1 H2O; spectrum -n20000 -k5000 -h0.002 -c2 -p H2O.cpa\n\
");
  exit(0); }

  initscroll(0);

  loop (iarg,1,narg)
    if (arg[iarg][0]=='-') {
      switch (arg[iarg][1]) {
        case 0: fn=NULL; break;
        case 'c': col=atoi(arg[iarg]+2)-1; break;
        case 'd': prec=atoi(arg[iarg]+2); break;
        case 'o': omit=atoi(arg[iarg]+2); break;
        case 'm': minpeak=atof(arg[iarg]+2); break;
        case 'n': n=atoi(arg[iarg]+2); break;
        case 'k': k=atoi(arg[iarg]+2); break;
        case 'w': win=atoi(arg[iarg]+2); break;
        case 'h': dt=atof(arg[iarg]+2); break;
        case 'p': plot++; break;
        case 'q': isq++; break;
        case 'v': isverlet++; break;
        case 'T': iscm=0; break;
        case 'e': slowdown=atof(arg[iarg]+2); break;
        default: ERROR(("%s is wrong argument",arg[iarg])); } }
    else
      fn=arg[iarg];

  {
    int sum=0,blk=n,sqblk,d;
    double q;

    while (blk>1) {
      sqblk=(int)sqrt(blk+0.5);
      loopto (d,2,sqblk) if (blk % d == 0) break;

      if (blk % d != 0) d=blk; /* blk = prime */

      sum+=d;

      blk=blk/d; }

    q=sum/log(n)/3; /* should be /e ideally */

    if (q>slowdown) WARNING(("%d is a product of not so small primes,\n\
*** estimated slowdown = %.3g (-eBIG to suppress this message)",n,q))
  }

  if (fn) {
    in=fopen(fn,"rt");
    if (!in) ERROR(("%s not found",fn))
    out=fopen(string("%s.sp",fn),"wt");
    if (dt && minpeak>=0) fpeak=fopen(string("%s.pk",fn),"wt"); }
  else {
    in=stdin;
    out=fpeak=stdout; }

  if (n<=0) ERROR(("no or bad # of data (option -n)"))
  if (!k) k=n/2;
  if (k>n) ERROR(("k>n: change options -k (or -n)"))

  alloczero(y,n*sizeof(y[0]));

  loop (i,-omit,n) {
    double x=2*PI*(i+0.5)/n,yy;
    char line[1024],*tok;

    do if (!fgets(line,1024,in)) {
      if (!missing) {
        fprintf(stderr,"\
only %d data lines on input (%d expected)\n\
WARNING: missing data will be padded by the average %.8g\n",i+omit,n+omit,sum/ndata);
        if (win) fprintf(stderr,"ERROR: -w specified => the window may not be properly centered\n\
(only a few padded data are acceptable without a loss of precision)\n"); }
      missing++; }
    while (line[0]==0 || strchr("!#",line[0]));

    if (i>=0) {
      if (missing)
        y[i].re=sum/ndata;
      else {
        tok=strtok(line," \t\n\r");

        loop (ic,0,col) {
          if (!tok) ERROR(("line %d not enough columns",i))
	  tok=strtok(NULL," \t\n\r"); }

        if (!tok) ERROR(("line %d not enough columns",i))

        yy=atof(tok);
        sum+=yy;
        ndata++;
        y[i].re=yy; }
      if (win) y[i].re*=window(x); } }

  Fourier(y,n);

  sp=(real*)y;

  loop (i,0,k) {
    if (dt) prt_("%.*g ",prec,conv(i/(dt*n)));
    sp[i]=(i>=2)*sqrt(Sqr(y[i].re)+Sqr(y[i].im));
    prt("%.*g",prec,sp[i]); }

  if (fn) {
    fclose(out);
    out=fpeak; }

  if (dt && out)
    loop (i,1,k-5)
      if (sp[i]<sp[i+1] && sp[i+1]<sp[i+2] && sp[i+2]>sp[i+3] && sp[i+3]>sp[i+4]) {
	real p=peak(sp+i,4);
	if (p && A>minpeak) prt("\n%.*g 0\n%.*g %.*g",
                   prec,conv((i+p)/(dt*n)),
                   prec,conv((i+p)/(dt*n)),
                   prec,A); }

  if (out) fclose(out);

  if (plot) {
    if (dt) i=system(string("plot %s.sp:1:2 %s.pk",fn,fn));
    else i=system(string("plot %s.sp:0:1",fn));
    if (i) {
      fprintf(stderr,"plot failed\n");
      return 1; } }

  return 0;
}
